clc
clear
close all

global xData yData yCalc glbRsqr QSPcoeff
zFilename = "besselj_0_x_halton_random_run2";
txtFile = strcat(zFilename, ".txt");
xlFile = strcat(zFilename, ".xlsx");
diary(txtFile)
gFile =  strcat(zFilename, ".jpg");
fprintf("%s\n", datetime(now,'ConvertFrom','datenum'));
format longE
sEqn = "besselj(0,x)";
fprintf("%s\n",sEqn);
fprintf("x=0:0.1:10\n")
xData= 0:0.1:10;
xData = xData';
n = length(xData);
yData = besselj(0,xData);
order = 6;
[Lb,Ub] = makeLimits(order, 0.5, 0.9, 0.1);

[bestX,bestFx] = haltonRandomSearch(@quantShammasPoly,Lb,Ub,1000000);

SSE = quantShammasPoly(bestX);
% calculate adjusted value of the coefficient of determination
glbRsqr = 1 - (1 - glbRsqr)*(n-1)/(n-order-1);
fprintf("Adjusted Rsqr = %f\n", glbRsqr);
fprintf("Quantum Shammas Polynomial Powers\n");
bestX
fprintf("Quantum Shammas Polynomial Coefficients\n");
QSPcoeff = QSPcoeff'
fprintf("\nRegular polynomial fit\n");
c = polyfit(xData,yData,order)
yPoly = polyval(c,xData);
r = rsqr(yData,yPoly);
% calculate adjusted value of the coefficient of determination
r = 1 - (1 - r)*(n-1)/(n-order-1);
fprintf("Adjusted Rsqr = %f\n", r);
fprintf("\nMultiple-half-power  polynomial fit\n");
c2 = polyfit(sqrt(xData),yData,order)
yPoly2 = polyval(c2,sqrt(xData));
r2 = rsqr(yData,yPoly2);
% calculate adjusted value of the coefficient of determination
r2 = 1 - (1 - r2)*(n-1)/(n-order-1);
fprintf("Adjusted Rsqr = %f\n", r2);

figure(1)
plot(xData,yData,xData,yCalc,xData,yPoly,xData,yPoly2);
title(sEqn)
xlabel("X")
ylabel("Y");
grid;
ax = gca;
exportgraphics(ax,gFile);

QSPpwr = bestX;
Coeff = flip(c);
HalfCoeff = flip(c2);
T1 = array2table(QSPpwr);
writetable(T1,xlFile,"Sheet","Sheet1","Range","A1");
T2 = array2table(QSPcoeff);
writetable(T2,xlFile,"Sheet","Sheet1","Range","A4");
T3 = array2table(Coeff);
writetable(T3,xlFile,"Sheet","Sheet1","Range","A7");
T4 = array2table(HalfCoeff);
writetable(T4,xlFile,"Sheet","Sheet1","Range","A10");
r_sqr = [glbRsqr r r2];
T5 = array2table(r_sqr);
writetable(T5,xlFile,"Sheet","Sheet1","Range","A13");

format short
diary off

function [Lb,Ub] = makeLimits(order, minPwr, maxPwr, diffPwr)
  Lb = zeros(1,order);
  Ub = zeros(1,order);
  Lb(1) = minPwr;
  Ub(1) = maxPwr;
  for i=2:order
    j = i - 1;
    Lb(i) = Ub(i-1)+diffPwr;
    Ub(i) = Lb(i) + maxPwr;
  end
end

function r = rsqr(y,ycalc)
  n = length(y);
  ymean = mean(y);
  SStot = sum((y - ymean).^2);
  SSE = sum((y - ycalc).^2);
  r = 1 - SSE / SStot;
end

